@@ -299,8 +299,10 @@ class getStripe:
                     unitsize = np.floor(chrsize/self.resol/itera)
                     unitsize = np.uint64(unitsize)
 
-                    bgleft = np.zeros((400, 0))
-                    bgright = np.zeros((400, 0))
+                    bgleft_up = np.zeros((400, 0))
+                    bgright_up = np.zeros((400, 0))
+                    bgleft_down = np.zeros((400, 0))
+                    bgright_down = np.zeros((400, 0))
 
                     n_pool = []
 
@@ -333,58 +335,87 @@ class getStripe:
                             del mat
                         elif len(pool) < sss:
                             randval = random.choices(pool, k=len(pool))
-                            tableft = np.zeros((400, len(pool)))
-                            tabcenter = np.zeros((400, len(pool)))
-                            tabright = np.zeros((400, len(pool)))
+                            tableft_up = np.zeros((400, len(pool)))
+                            tabcenter_up = np.zeros((400, len(pool)))
+                            tabright_up = np.zeros((400, len(pool)))
+                            tableft_down = np.zeros((400, len(pool)))
+                            tabcenter_down = np.zeros((400, len(pool)))
+                            tabright_down = np.zeros((400, len(pool)))
 
                             for i in range(len(pool)):
                                 x = randval[i]
                                 for j in range(0,400):
-                                    det = np.random.choice([0,1])
-                                    if det == 0:
-                                        y = x + j
-                                    else:
-                                        y = x - j
+                                    #det = np.random.choice([0,1])
+                                    y_down = x + j
+                                    y_up = x - j
+
+                                    #if det == 0:
+                                    #    y = x + j
+                                    #else:
+                                    #    y = x - j
                                     if it > 0 :
-                                        y = y + 400
+                                        y_down = y_down + 400
+                                        y_up = y_up + 400
 
+                                    tableft_up[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y_up - background_up):(y_up + background_down)])
+                                    tabcenter_up[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y_up - background_up):(y_up + background_down)])
+                                    tabright_up[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y_up - background_up):(y_up + background_down)])
 
-                                    tableft[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y - background_up):(y + background_down)])
-                                    tabcenter[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y - background_up):(y + background_down)])
-                                    tabright[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y - background_up):(y + background_down)])
+                                    tableft_down[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y_down - background_up):(y_down + background_down)])
+                                    tabcenter_down[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y_down - background_up):(y_down + background_down)])
+                                    tabright_down[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y_down - background_up):(y_down + background_down)])
+
+                            bgleft_up_temp = np.subtract(tabcenter_up, tableft_up)
+                            bgright_up_temp = np.subtract(tabcenter_up, tabright_up)
+                            bgleft_up = np.column_stack((bgleft_up, bgleft_up_temp))
+                            bgright_up = np.column_stack((bgright_up, bgright_up_temp))
+                            bgleft_down_temp = np.subtract(tabcenter_down, tableft_down)
+                            bgright_down_temp = np.subtract(tabcenter_down, tabright_down)
+                            bgleft_down = np.column_stack((bgleft_down, bgleft_down_temp))
+                            bgright_down = np.column_stack((bgright_down, bgright_down_temp))
 
-                            bgleft_temp = np.subtract(tabcenter, tableft)
-                            bgright_temp = np.subtract(tabcenter, tabright)
-                            bgleft = np.column_stack((bgleft, bgleft_temp))
-                            bgright = np.column_stack((bgright, bgright_temp))
                             del mat
                         else:
                             randval = random.choices(pool, k=sss)
-                            tableft = np.zeros((400, sss))
-                            tabcenter = np.zeros((400, sss))
-                            tabright = np.zeros((400, sss))
+                            tableft_up = np.zeros((400, sss))
+                            tabcenter_up = np.zeros((400, sss))
+                            tabright_up = np.zeros((400, sss))
+                            tableft_down = np.zeros((400, sss))
+                            tabcenter_down = np.zeros((400, sss))
+                            tabright_down = np.zeros((400, sss))
 
                             for i in range(sss):
                                 x = randval[i]
                                 for j in range(0, 400):
-                                    det = np.random.choice([0,1])
-                                    if det == 0:
-                                        y = x + j
-                                    else:
-                                        y = x - j
+                                    y_down = x + j
+                                    y_up = x - j
+                                   # det = np.random.choice([0,1])
+                                   # if det == 0:
+                                   #     y = x + j
+                                   # else:
+                                   #     y = x - j
                                     if it > 0:
-                                        y = y + 400
-                                    tableft[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y - background_up):(y + background_down)])
-                                    tabcenter[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y - background_up):(y + background_down)])
-                                    tabright[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y - background_up):(y + background_down)])
+                                        y_up = y_up + 400
+                                        y_down = y_down + 400
+                                    tableft_up[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y_up - background_up):(y_up + background_down)])
+                                    tabcenter_up[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y_up - background_up):(y_up + background_down)])
+                                    tabright_up[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y_up - background_up):(y_up + background_down)])
+                                    tableft_down[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y_down - background_up):(y_down + background_down)])
+                                    tabcenter_down[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y_down - background_up):(y_down + background_down)])
+                                    tabright_down[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y_down - background_up):(y_down + background_down)])
+
+                            bgleft_up_temp = np.subtract(tabcenter_up, tableft_up)
+                            bgright_up_temp = np.subtract(tabcenter_up, tabright_up)
+                            bgleft_down_temp = np.subtract(tabcenter_down, tableft_down)
+                            bgright_down_temp = np.subtract(tabcenter_down, tabright_down)
 
-                            bgleft_temp = np.subtract(tabcenter, tableft)
-                            bgright_temp = np.subtract(tabcenter, tabright)
                             del mat
-                            bgleft = np.column_stack((bgleft, bgleft_temp))
-                            bgright = np.column_stack((bgright, bgright_temp))
+                            bgleft_up = np.column_stack((bgleft_up, bgleft_up_temp))
+                            bgright_up = np.column_stack((bgright_up, bgright_up_temp))
+                            bgleft_down = np.column_stack((bgleft_down, bgleft_down_temp))
+                            bgright_down = np.column_stack((bgright_down, bgright_down_temp))
 
-                    depl = int(ss - bgleft.shape[1])
+                    depl = int(ss - bgleft_up.shape[1])
                     if depl > 0:
                         rich = np.argmax(n_pool)
                         test_region_start1 = int(unitsize * self.resol * rich+1)
@@ -410,46 +441,65 @@ class getStripe:
                         pool = [x for x in list(range(nrow)) if x not in zeroindex[0].tolist()]
                         pool = [x for x in pool if x > 20 and x < (unitsize - 20)]
                         randval = random.choices(pool, k=depl)
-                        tableft = np.zeros((400, depl))
-                        tabcenter = np.zeros((400, depl))
-                        tabright = np.zeros((400, depl))
-
+                        tableft_up = np.zeros((400, depl))
+                        tabcenter_up = np.zeros((400, depl))
+                        tabright_up = np.zeros((400, depl))
+                        tableft_down = np.zeros((400, depl))
+                        tabcenter_down = np.zeros((400, depl))
+                        tabright_down = np.zeros((400, depl))
                         for i in range(depl):
                             x = randval[i]
                             for j in range(0, 400):
-                                det = np.random.choice([0, 1])
-                                if det == 0:
-                                    y = x + j
-                                else:
-                                    y = x - j
+                                y_down = x + j
+                                y_up= x - j
+                                #det = np.random.choice([0, 1])
+
+                                #if det == 0:
+                                #    y = x + j
+                                #else:
+                                #    y = x - j
                                 if it > 0:
-                                    y = y + 400
-                                    tableft[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y - background_up):(y + background_down)])
-                                    tabcenter[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y - background_up):(y + background_down)])
-                                    tabright[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y - background_up):(y + background_down)])
+                                    y_up = y_up + 400
+                                    y_down = y_down+400
+                                    tableft_up[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y_up - background_up):(y_up + background_down)])
+                                    tabcenter_up[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y_up - background_up):(y_up + background_down)])
+                                    tabright_up[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y_up - background_up):(y_up + background_down)])
+                                    tableft_down[j, i] = np.mean(mat[(x - background_up - background_size):(x - background_up), (y_down - background_up):(y_down + background_down)])
+                                    tabcenter_down[j, i] = np.mean(mat[(x - background_up):(x + background_down), (y_down - background_up):(y_down + background_down)])
+                                    tabright_down[j, i] = np.mean(mat[(x + background_down):(x + background_down + background_size), (y_down - background_up):(y_down + background_down)])
+
+                        bgleft_up_temp = np.subtract(tabcenter_up, tableft_up)
+                        bgright_up_temp = np.subtract(tabcenter_up, tabright_up)
+                        bgleft_down_temp = np.subtract(tabcenter_down, tableft_down)
+                        bgright_down_temp = np.subtract(tabcenter_down, tabright_down)
 
-                        bgleft_temp = np.subtract(tabcenter, tableft)
-                        bgright_temp = np.subtract(tabcenter, tabright)
                         del mat
-                        bgleft = np.column_stack((bgleft, bgleft_temp))
-                        bgright = np.column_stack((bgright, bgright_temp))
+                        bgleft_up = np.column_stack((bgleft_up, bgleft_up_temp))
+                        bgright_up = np.column_stack((bgright_up, bgright_up_temp))
+                        bgleft_down = np.column_stack((bgleft_down, bgleft_down_temp))
+                        bgright_down = np.column_stack((bgright_down, bgright_down_temp))
 
-                        return bgleft, bgright
+                return bgleft_up, bgright_up, bgleft_down, bgright_down
             # apply parallel.
             print('2. Constituting background ... \n')
             result = Parallel(n_jobs=self.core)(delayed(main_null_calc)(chr) for chr in tqdm(chromnames2))
-            bgleft = np.zeros((400,0))
-            bgright = np.zeros((400,0))
+            bgleft_up = np.zeros((400,0))
+            bgright_up = np.zeros((400,0))
+            bgleft_down = np.zeros((400,0))
+            bgright_down = np.zeros((400,0))
+
             for i in range(len(result)):
                 if(type(result[i]) == type(None)):
                     continue
                 else:
-                    bl,br = result[i]
-                    bgleft=np.column_stack((bgleft,bl))
-                    bgright=np.column_stack((bgright,br))
+                    blu,bru,bld,brd = result[i]
+                    bgleft_up=np.column_stack((bgleft_up,blu))
+                    bgright_up=np.column_stack((bgright_up,bru))
+                    bgleft_down=np.column_stack((bgleft_down,bld))
+                    bgright_down=np.column_stack((bgright_down,brd))
 
             print('Elapsed time for background estimation: ' + str(np.round((time.time() - t_background_start) / 60, 3)) + ' min')
-            return bgleft, bgright
+            return bgleft_up, bgright_up, bgleft_down, bgright_down
 
     def getMean(self, df, mask='0'):
 
@@ -486,7 +536,7 @@ class getStripe:
             listS[i] = SUM
         return listM,listS
 
-    def pvalue(self, bgleft, bgright, df):
+    def pvalue(self, bgleft_up, bgright_up, bgleft_down, bgright_down, df):
         background_size = 50000 / self.resol
         background_size = int(background_size)
 
@@ -535,13 +585,19 @@ class getStripe:
             for j in range(len(center)):
                 if x1 == y1:  # downward stripe
                     difference = j
+                    if difference >= 400:
+                        difference = 399
+                    difference = int(difference)
+                    bleft = bgleft_down[difference,:]
+                    bright = bgright_down[difference,:]
                 elif x2 == y2:  # upward stripe
                     difference = y2 - y1 - j - 1
-                if difference >= 400:
-                    difference = 399
-                difference = int(difference)
-                bleft = bgleft[difference, :]
-                bright = bgright[difference, :]
+                    if difference >= 400:
+                        difference = 399
+                    difference = int(difference)
+                    bleft = bgleft_up[difference,:]
+                    bright = bgright_up[difference,:]
+
                 p1 = len(np.where(bleft >= left_diff[j])[0]) / len(bleft[np.where(~np.isnan(bleft))])
                 p2 = len(np.where(bright >= right_diff[j])[0]) / len(bright[np.where(~np.isnan(bright))])
                 pval = max(p1, p2)
@@ -781,7 +837,7 @@ class getStripe:
                 listTotal[i] = cT
         return listg,listMean,listTotal
 
-    def extract(self, MP, index, perc , bgleft, bgright):
+    def extract(self, MP, index, perc , bgleft_up, bgright_up, bgleft_down, bgright_down):
         with np.errstate(divide='ignore', invalid='ignore'):
             def search_frame(idx):
                 start = idx * 200 - 100
@@ -844,7 +900,7 @@ class getStripe:
 
             # Stripe filtering and scoring
             # res2 = self.scoringstripes(res)
-            p = self.pvalue(bgleft, bgright, res)
+            p = self.pvalue(bgleft_up, bgright_up, bgleft_down, bgright_down, res)
             #s = self.scoringstripes(res)
             res = res.assign(pvalue=pd.Series(p))
             #res = res.assign(Stripiness=pd.Series(s[5]))
